/* jshint esversion: 6, node: true, unused: true */
'use strict';

const helper  = require('./helper');
const ready   = require('./ready');
const app     = require('electron').app;
const path    = require('path').join(app.getPath('userData'), 'config.json');
const winston = require('winston');

const defaults = Object.freeze({
  forceQuit:      false,
  boxName:        undefined,
  username:       undefined,
  uid:            undefined,
  apiToken:       undefined,
  scanInterval:   10,
  folderName:     undefined
});
var config = {};
var json = '{}';

// assigns values from left to right into the config object
function configure(data) {
  var custom = JSON.parse(data);
  Object.assign(config, defaults, custom);
  json = data;
  winston.debug('[config] Configured.', config);
}

// stores JSON representation for given object asynchronously
function persist() {
  json = JSON.stringify(config);
  winston.debug('[config] writing...', json);
  helper.write(path, json);
}

// assigns multiple values before data is written
function assign(object) {
  Object.assign(config, object);
  persist(config);
}

// deletes all configuration changes
function reset() {
  Object.assign(config, defaults);
  persist(config);
}

// The initialization should include reading the configuration.
ready.when(function(resolve, reject) {
  winston.debug('[config] Reading...', { path: path });

  helper.read(path).
    catch(function() { return '{}'; }).
    then(configure).
    then(resolve, reject);
});

// To ensure we save changes immediately and prevent direct access to the configuration object we export a proxy.
module.exports = new Proxy(config, {
  // extranous methods on config proxy object
  get: function(target, property) {
    switch (property) {
      case 'json': return json;
      case 'reset': return reset;
      case 'assign': return assign;
      default: return target[property];
    }
  },
  // persist changes aynchronously as soon has they happen
  set: function(target, property, value) {
    if (target[property] !== value) {
      target[property] = value;
      persist(target);
    }

    return true;
  }
});
