/* jshint esversion: 6, node: true, unused: true */
'use strict';

const moduleName = 'native/darwin';

const app         = require('app');
const execFile    = require('child_process').execFile;
const path        = require('path');
const winston     = require('winston');

const appPath       = app.getAppPath();
const resourcePath  = path.resolve(appPath, '..');
const dotAppPath    = path.resolve(resourcePath, '../..');
const dotAppName    = path.basename(dotAppPath, '.app');

function osascript(code) {
  return new Promise(function(resolve, reject) {
    winston.debug(`[${ moduleName }] osascript`, { eval: code });
    execFile('osascript', [`-e${code}`], function(error, stdout, stderr) {
      if (error) reject(stderr.toString(), error, stdout);
      else resolve(stdout, stderr);
    });
  }).catch(function(error) {
    winston.error(`[${ moduleName }] osascript`, { error: error });
  });
}

function loginItems() {
  return osascript('tell application "System Events" to get the name of every login item').
  then(function(stdout) {
    var data  = stdout.toString();
    var items = data.split(',').map(function(i) { return i.trim(); });

    return Promise.resolve(items);
  });
}
function enableLaunchAtLogin() {
  return osascript(`tell application "System Events" to make login item at end with properties {path:"${ dotAppPath }", hidden:false}`);
}
function disableLaunchAtLogin() {
  return osascript(`tell application "System Events" to delete login item "${ dotAppName }"`);
}

function isInList(apps) {
  return apps.some(function(app) { return app === dotAppName; });
}

function toggleLaunchAtLogin() {
  return loginItems().then(function(apps) {
    return isInList(apps) ? disableLaunchAtLogin() : enableLaunchAtLogin();
  }).catch(function() {
    winston.error(`[${ moduleName }] An error occured!`, { arguments: arguments });
  });
}

var isLaunchedAtLogin = loginItems().then(function(apps) {
  return Promise.resolve(isInList(apps));
}, function() {
  return Promise.resolve(false);
});

function getTrayImagePath(status) {
  if (status === undefined) status = 0;
  return `${appPath}/native/darwin/icons/icon${status}Template.png`;
}

module.exports = function() {
  this.toggleLaunchAtLogin = toggleLaunchAtLogin;
  this.isLaunchedAtLogin   = isLaunchedAtLogin;
  this.getTrayImagePath    = getTrayImagePath;
  this.downloadURL = 'http://packages2.protonet.info/protosync/current/ProtoSync.dmg';

  var servicePath = path.resolve(app.getAppPath(), '../../Library/Services/ProtoSync Finder Integration.service');
  var service = execFile('open', [servicePath], function(error, stdout, stderr) {
    if (stdout) winston.info(stdout);
    if (error) winston.error(stderr);
  });
  app.on('quit', function() { service.kill(); });

  return this;
};
