/* jshint esversion: 6, node: true, unused: true */
'use strict';

const util         = require('util');
const EventEmitter = require('events');
const winston      = require('winston');
const helper       = require('./helper');

function StatusAgent(url) {
  EventEmitter.call(this);

  var emitter = this;
  var since, destroyed = false;

  function handleResponse(events) {
    if (destroyed) return Promise.resolve();

    events.forEach(function(event) {
      winston.info(`[status-agent#${event.id}] ${event.type}`, event.data);
      try {
        emitter.emit(event.type, event.data);
      } catch (e) {
        return Promise.reject(e);
      }
    });

    if (events.length > 0) since = events.slice(-1)[0].id;

    return helper.getJSON(`${url}?since=${since}`).then(handleResponse);
  }

  function restart(at) {
    since = at || 0;
    helper.getJSON(`${url}?since=${ since }`).then(handleResponse).
      catch(function(error) {
        winston.error('[status-agent] An error occured.', { error: error });
        setTimeout(function() { restart(since); }, 1000);
      });
  }
  restart();

  function destroy() { destroyed = true; }
  this.destroy = destroy;
}
util.inherits(StatusAgent, EventEmitter);

module.exports = StatusAgent;
