/* jshint esversion: 6, jquery: true, node: true, unused: true */
/* globals app: true, btoa: true  */
'use strict';

app.controller("LoginCtrl", function($scope, $http, $q, $timeout) {
  const electron  = require('electron');
  const ipc       = electron.ipcRenderer;
  const remote    = require('remote');
  const protosync = remote.app;
  const path      = require('path');
  const home      = protosync.getPath('home');
  const shell     = require("shell");
  const syncPath  = path.join(home, protosync.getName());


  $scope.page = "boxname";
  $scope.host = "";
  $scope.boxName = "";
  $scope.email = "";
  $scope.password = "";

  $scope.checkHost = function() {
    $scope.host = parseHost($scope.boxName);
    if ($scope.host.length < 6) {
      return;
    }

    $scope.checking = true;
    $scope.hostExists = false;

    checkHost($scope.host).then(function() {
      $scope.hostExists = true;
    }, function() {
      $scope.hostExists = false;
    }).finally(function() {
      $scope.checking = false;
    });
  };

  const Menu = remote.Menu;

  var ctxMenuTemplate = [{
    label: 'Paste',
    accelerator: 'CmdOrCtrl+V',
    role: 'paste'
  }];
  var ctxMenu = Menu.buildFromTemplate(ctxMenuTemplate);

  $scope.showContextMenu = function() {
    var currentWindow = remote.getCurrentWindow();
    ctxMenu.popup(currentWindow);
  };

  function checkHost(host) {
        return $http.head(`https://${ host }/robots.txt`);
  }

  function parseHost(boxName) {
    return $.trim(boxName)
      .replace(/https?\:\/\//, "")
      .replace(/\/.+/, "")
      .replace(/\.protonet\.de$/, ".protonet.info")
      .toLowerCase();
  }

  function getMe() {
    var deferred = $q.defer();
    $http.get("https://" + $scope.host + "/api/v1/me", {
      headers: {
        "X-Requested-With": "XMLHttpRequest",
        "Authorization":    "Basic " + btoa($scope.email + ":" + $scope.password)
      }
    }).then(function(response) {
      deferred.resolve(response.data.me);
    }).catch(function(response) {
      deferred.reject(response);
    });
    return deferred.promise;
  }

  $scope.next = function() {
    if (!$scope.hostExists) {
      return;
    }
    $scope.page = "login";
  };

  $scope.login = function() {
    $scope.error = null;
    $scope.loading = true;
    getMe().then(function(user) {
      if (user.role === "guest") {
        // TODO: Nicer error message
        $scope.error = "You have the guest role. Guests cannot use sync functionality.";
        return;
      }

      var data = {
        boxname:  $scope.host,
        protosyncthingpath: syncPath,
        username: user.username,
        email:    user.email,
        password: $scope.password
      };

      ipc.send("credentials", JSON.stringify(data));
      $scope.page = "done";
    }, function() {
      $scope.loading = false;
      $scope.error = "Incorrect email or password.";
    });
  };

  $scope.openSyncFolder = function() {
    shell.openItem(syncPath);
    remote.getCurrentWindow().close();
  };

  $scope.back = function() {
    $scope.backwards = true;
    $scope.page = "boxname";
    $timeout(function() {
      $scope.backwards = false;
    }, 600);
  };

  $scope.openPasswordForgotten = function() {
    var url = "https://" + $scope.host + "/users/password/new";
    shell.openExternal(url);
  };

});
